import logging
import os
import configparser

logger = logging.getLogger(__name__)

config_dir = os.path.expanduser('~/.python-magnatune/')
config_file = os.path.join(config_dir, 'config.ini')


def check_config_dir():
    """Creates the config dir is needed."""
    if not os.path.exists(config_dir):
        os.mkdir(config_dir)


class ConfigArgs:
    AUTHORIZED_OPTIONS = ('verbose', 'format', 'dlformat', 'login', 'extract')

    def __init__(self, args):
        self.args = args
        self.config = configparser.ConfigParser()
        try:
            self.config.read(config_file)
        except configparser.Error as e:
            logger.warning("Error while reading the config file, ignoring :\n%s", e)

        for option in self.config['default']:
            if option not in ConfigArgs.AUTHORIZED_OPTIONS:
                logger.warning('Option "%s" in config file will be ignore', option)

    def __getattr__(self, item):
        arg = getattr(self.args, item)
        if not arg:
            try:
                if item in ConfigArgs.AUTHORIZED_OPTIONS:
                    arg = self.config['default'][item]
            except KeyError:
                pass
        return arg



import unittest
import mock


class MockDbm(dict):
    def close(self):
        pass


class TestXMLDownload(unittest.TestCase):
    def patch(self, *args, **kwargs):
        patcher = mock.patch(*args, **kwargs)
        mock_object = patcher.start()
        self.addCleanup(patcher.stop)
        return mock_object

    def setUp(self):
        self.download = self.patch('magnatune.api.download')

        self.dbm = MockDbm()
        open = self.patch('dbm.open')
        open.return_value = self.dbm

        self.server_time = 123456789
        time = self.patch('time.time')
        time.return_value = self.server_time

        self.server_crc = 'servercrc'
        self.urlopen = self.patch('urllib.request.urlopen')
        self.urlopen().read.return_value = self.server_crc
        self.urlopen.reset_mock()

        self.patch('magnatune.config.check_config_dir')

    def test_non_existing_file(self):
        from magnatune.api import update_if_needed
        update_if_needed()

        self.download.assert_called_once_with()

    def test_recent_update(self):
        self.dbm['updated'] = '123456788.42'

        from magnatune.api import update_if_needed
        update_if_needed()

        self.assertFalse(self.download.called)

    def test_crc_unchanged(self):
        self.dbm['crc'] = self.server_crc
        self.dbm['updated'] = 1

        from magnatune.api import update_if_needed, CRC_URL
        update_if_needed()

        self.urlopen.assert_called_once_with(CRC_URL)
        self.assertFalse(self.download.called)

    def test_download_needed(self):
        self.dbm['crc'] = self.server_crc + 'changed'
        self.dbm['updated'] = 1

        from magnatune.api import update_if_needed, CRC_URL
        update_if_needed()

        self.download.assert_called_once_with()
        self.urlopen.assert_called_once_with(CRC_URL)
        self.assertEqual(self.dbm['crc'], self.server_crc)
        self.assertEqual(self.dbm['updated'], str(self.server_time))



import unittest
import mock


class TestSearchAlbum(unittest.TestCase):
    def setUp(self):
        patcher = mock.patch('magnatune.api.get_database')
        self.db = patcher.start()()
        self.addCleanup(patcher.stop)

    def test_searching_with_all_params(self):
        album = mock.Mock()
        self.db.getroot().Album = [album]

        album.find().text.lower.return_value = mock.MagicMock()
        album.find().text.lower().__contains__.return_value = True

        attrs = ('artist', 'albumname', 'magnatunegenres', 'artistdesc')
        call = {att: att + 'called' for att in attrs}

        from magnatune.search import search_album
        list(search_album(**call))

        self.assertEqual(album.find().text.lower().__contains__.call_args_list,
            [((value,), {}) for value in call.values()])

    def test_wrong_arg(self):
        from magnatune.search import search_album
        with self.assertRaises(TypeError):
            list(search_album(ratatouille='plonk'))


class TestAuthUrl(unittest.TestCase):
    def test_adding_the_auth_to_urls_works(self):
        from magnatune.search import auth_url

        data = [('http://he3.magnatune.com/all/'
                 '01-Ouverture%20Le%20bourgeois%20gentilhomme%20(JB%20Lully)-'
                 'The%20Bach%20Players.{0}'.format(ext),
                 'http://tagada:test@download.magnatune.com/all/'
                 '01-Ouverture%20Le%20bourgeois%20gentilhomme%20(JB%20Lully)-'
                 'The%20Bach%20Players_nospeech.{0}'.format(ext))
        for ext in ('mp3', 'ogg')]

        for input, output in data:
            self.assertEqual(auth_url(input, 'tagada:test'), output)



# -*- coding: utf-8 -*-

import time
import urllib.request
import os
import dbm
import bz2
import lxml.objectify
from magnatune.config import config_dir, check_config_dir

import logging

logger = logging.getLogger(__name__)

CRC_URL = 'http://he3.magnatune.com/info/changed.txt'
ALBUM_INFO_URL = 'http://he3.magnatune.com/info/album_info_xml.bz2'

db_file = os.path.join(config_dir, 'api.db')
album_info_file = os.path.join(config_dir, 'album_info.xml')


def download():
    """
    Downloads the last version of the api file from the server.
    """
    source = urllib.request.urlopen(ALBUM_INFO_URL)
    with open(album_info_file, 'wb') as f:
        f.write(bz2.decompress(source.read()))


def update_if_needed():
    """
    Checks if an update of the api file if needed, and download it if needed.
    """
    check_config_dir()
    db = dbm.open(db_file, 'c')
    try:
        updated = float(db.get('updated', 0))
        if time.time() - updated < 60 * 60 * 24:  #24 hours
            logger.debug(
                'Database file updated less than 24 hours, not updating.')
            return

        crc = db.get('crc', None)
        logger.info('Updating CRC file')
        new_crc = urllib.request.urlopen(CRC_URL).read()
        db['updated'] = str(time.time())
        if crc == new_crc:
            logger.debug('Database file up-to-date.')
            return

        logger.info('Updating database file.')
        download()
        db['crc'] = new_crc
    finally:
        db.close()


_db = None


def get_database():
    """
    Returns the api database as an lxml.objectify object, updating the
    database if needed.
    """
    global _db
    if not _db:
        update_if_needed()
        _db = lxml.objectify.parse(album_info_file)
    return _db



import argparse
import logging
import magnatune.search
import magnatune.config

FORMATS = {'ogg': 'oggurl',
           'mp3': 'url',
           'mp3lofi': 'mp3lofi'}


def main():
    parser = argparse.ArgumentParser(description="Search an album.")

    verbose = parser.add_mutually_exclusive_group()
    verbose.add_argument('--verbose', '-v', action='store_true',
                         help='Print informative output.')
    verbose.add_argument('--quiet', '-q', action='store_true',
                         help='Supress all non-warning informational output.')

    action = parser.add_mutually_exclusive_group()
    action.add_argument('--stream', '-s', action='store_true',
                        help='Output the streaming url of the track.')

    stream = parser.add_argument_group('Streaming options')
    stream.add_argument('--format', '-f', nargs='?', choices=FORMATS.keys(),
                        help='The format to use for streaming url.')

    action.add_argument('--download', '-d', action='store_true',
                        help='Download the albums.')

    download = parser.add_argument_group('Download options')
    download.add_argument('--dlformat', nargs='?',
                        choices=('web', 'wav', '128kmp3', 'ogg', 'vbr', 'flac'),
                        help='The format to use for downloading albums.')
    download.add_argument('--extract', '-e', nargs='?', const='.',
                        help='Extract downloaded albums to path (. by default).')

    parser.add_argument('--login', '-l',
                        help='The magnatune login and password in the '
                        '"login:passwd" format.')


    search = parser.add_argument_group('Search arguments')
    search.add_argument('--artist', '-a', help='Filter by artist name.')
    search.add_argument('--albumname', '-n', help='Filter by album name.')
    search.add_argument('--artistdesc', help='Filter by artist description.')
    search.add_argument('--genre', '-g', help='Filter by genre.')

    args = magnatune.config.ConfigArgs(parser.parse_args())

    if args.verbose:
        loglevel = logging.DEBUG
    elif args.quiet:
        loglevel = logging.WARN
    else:
        loglevel = logging.INFO
    logging.basicConfig(level=loglevel)

    if not (args.artist or args.albumname or args.genre or args.artistdesc):
        parser.error('no search filter given')

    if args.download and not args.login:
        parser.error('cannot download an album without subscription login')

    for a in magnatune.search.search_album(artist=args.artist,
                                           albumname=args.albumname,
                                           magnatunegenres=args.genre,
                                           artistdesc=args.artistdesc):
        if args.stream:
            format = FORMATS[args.format]
            for t in a.Track:
                print(magnatune.search.stream_url(t, format, args.login))
        elif args.download:
            magnatune.search.download(a.albumsku, args.dlformat, args.extract, args.login)
        else:
            print(a.artist, '--', a.albumname)



from setuptools import setup, find_packages


setup(
    name="magnatune",
    version="0.2",
    packages=find_packages(),

    author="Georges Dubus",
    author_email="georges.dubus@compiletoi.net",
    description="Command line utility and lib to interact with the music "
                "website magnatune",
    long_description=open('README.rst', 'rt').read(),
    url="https://github.com/madjar/python-magnatune",
    license="GPLv3",
    keywords="magnatune music stream",

    classifiers = [
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Intended Audience :: End Users/Desktop",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Multimedia",
        ],


    install_requires=['lxml'],
    tests_require=['nose', 'Mock'],
    entry_points={'console_scripts': ['magnatune = magnatune.main:main']}
)



import sys
import logging
import urllib.request
import webbrowser
import zipfile
import lxml.etree
import magnatune.api


logger = logging.getLogger(__name__)


HANDLED_ALBUM_ATTRS = {'artist', 'albumname', 'magnatunegenres', 'artistdesc'}


def search_album(**kw):
    """Searchs for albums matching the arguments."""
    if not HANDLED_ALBUM_ATTRS.issuperset(kw):
        raise TypeError(
            'unhandled search on attributes : {0}'
            .format(', '.join(set(kw.keys()).difference(HANDLED_ALBUM_ATTRS))))

    db = magnatune.api.get_database()
    for a in db.getroot().Album:
        for attr, value in kw.items():
            if value and not value.lower() in a.find(attr).text.lower():
                break
        else:
            yield a


def auth_url(url, login):
    """Adds authentification to a streaming url"""
    return (url.replace('http://he3', 'http://{}@download'.format(login))
            .replace('.mp3', '_nospeech.mp3').replace('.ogg', '_nospeech.ogg'))


def stream_url(track, format=None, login=None):
     """Returns a streaming url for given track, in given format with given login."""
     if not format:
         format = 'ogg'
     url = str(track.find(format))
     if login:
        return auth_url(url, login)
     else:
         return url


def download(sku, format=None, extract=False, login=None):
    """Downloads the album with given sku id in given format, with given login.

    If extract is not empty, also extract to the given path."""
    if not format:
        format = 'web'
    # Preparation for the login
    user, pwd = login.split(':')
    password_manager = urllib.request.HTTPPasswordMgrWithDefaultRealm()
    password_manager.add_password(None, 'https://download.magnatune.com', user, pwd)
    auth_manager = urllib.request.HTTPBasicAuthHandler(password_manager)
    opener = urllib.request.build_opener(auth_manager)

    # Download of the information file
    url = ("http://download.magnatune.com/buy/"
           "membership_free_dl_xml?id=python&sku={}".format(sku))
    logger.debug("Downloading %s", url)
    content = opener.open(url).read()
    content = content.replace(b'<br>', b'')  # Workaround because the xml is malformated
    response = lxml.etree.fromstring(content)

    if format == 'web':
        url = response.find('DL_PAGE').text
        print(url)
        webbrowser.open(url)
    else:
        urlzip = response.find('URL_{}ZIP'.format(format.upper()))
        if urlzip is None:
            raise Exception('Unknown download format : %s' % format)

        filename = urlzip.text.split('/')[-1]
        logger.debug("Downloading %s", urlzip.text)
        with opener.open(urlzip.text) as source, open(filename, 'wb') as dest:
            headers = source.info()
            width = 32
            bs = 1024*8
            blocknum = 0
            size = int(headers.get("Content-Length", -1))
            output = logging.INFO >= logger.getEffectiveLevel()
            while 1:
                block = source.read(bs)
                if not block:
                    break
                dest.write(block)
                blocknum += 1
                if output and blocknum % 16 == 0:
                    cur = blocknum * bs
                    x = (width + 1) * cur // size
                    bar = '{} [{}{}] {:.1f}/{:.1f}\r'.format(filename, '='*x,
                                                             '-'*(width-x),
                                                             cur/(1024*1024),
                                                             size/(1024*1024))
                    sys.stderr.write(bar)
                    sys.stderr.flush()
            if output:
                sys.stderr.write('\n')

        if extract:
            logger.info("Extracting %s", filename)
            zipfile.ZipFile(filename).extractall(path=extract)
